#ifndef NNF_FLUXES_H
#define NNF_FLUXES_H

#include <nuflux/LegacyConventionalFlux.h>
#include <nuflux/LegacyPromptFlux.h>
#ifndef NO_PHOTOSPLINE
#include <nuflux/SplineFlux.h>
#include <nuflux/SplineFlux2.h>
#endif
#endif
